@@ -1,12 +1,14 @@
 """
 author: Maikel Punie <maikel.punie@gmail.com>
 """
+from __future__ import annotations
 
 import json
 import string
 
 from velbusaio.command_registry import commandRegistry
 from velbusaio.const import (
+    DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_TEMPERATURE,
     ENERGY_KILO_WATT_HOUR,
     ENERGY_WATT_HOUR,
@@ -26,9 +28,6 @@ class Channel:
     This is the basic abstract class of a velbus channel
     """
 
-    _on_status_update = None
-    _name_parts = {}
-
     def __init__(self, module, num, name, nameEditable, writer, address):
         self._num = num
         self._module = module
@@ -39,29 +38,31 @@ class Channel:
             self._is_loaded = False
         self._writer = writer
         self._address = address
+        self._on_status_update = []
+        self._name_parts = {}
 
-    def get_module_type(self):
+    def get_module_type(self) -> str:
         return self._module.get_type()
 
-    def get_module_type_name(self):
+    def get_module_type_name(self) -> str:
         return self._module.get_type_name()
 
-    def get_module_serial(self):
+    def get_module_serial(self) -> str:
         return self._module.get_serial()
 
-    def get_module_address(self):
+    def get_module_address(self) -> int:
         return self._module._address
 
-    def get_module_sw_version(self):
+    def get_module_sw_version(self) -> str:
         return self._module.get_sw_version()
 
-    def get_channel_number(self):
+    def get_channel_number(self) -> int:
         return self._num
 
-    def get_full_name(self):
+    def get_full_name(self) -> str:
         return f"{self._module.get_name()} ({self._module.get_type_name()})"
 
-    def is_loaded(self):
+    def is_loaded(self) -> bool:
         """
         Is this channel loaded
 
@@ -69,34 +70,49 @@ class Channel:
         """
         return self._is_loaded
 
-    def get_name(self):
+    def is_counter_channel(self) -> bool:
+        return False
+
+    def is_temperature(self) -> bool:
+        return False
+
+    def get_name(self) -> str:
         """
         :return: the channel name
         """
         return self._name
 
-    def set_name_part(self, part, name):
+    def set_name_part(self, part, name) -> None:
         """
         Set a part of the channel name
         """
         # if int(part) not in self._name_parts:
         #    return
         self._name_parts[int(part)] = name
-        if int(part) == 3:
+        if len(self._name_parts) == 3:
             self._generate_name()
 
-    def _generate_name(self):
+    def _generate_name(self) -> None:
         """
         Generate the channel name if all 3 parts are received
         """
         name = self._name_parts[1] + self._name_parts[2] + self._name_parts[3]
         self._name = "".join(filter(lambda x: x in string.printable, name))
         self._is_loaded = True
-        self._name_parts = None
+        self._name_parts = {}
 
     def __getstate__(self):
         d = self.__dict__
-        return {k: d[k] for k in d if k != "_writer" and k != "_on_status_update"}
+        return {
+            k: d[k]
+            for k in d
+            if k != "_writer" and k != "_on_status_update" and k != "_name_parts"
+        }
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        self._on_status_update = []
+        self._name_parts = {}
 
     def __repr__(self):
         items = []
@@ -108,24 +124,24 @@ class Channel:
     def __str__(self):
         return self.__repr__()
 
-    def update(self, data):
+    async def update(self, data: dict) -> None:
         """
         Set the attributes of this channel
         """
         for key, val in data.items():
             setattr(self, f"_{key}", val)
-        if self._on_status_update:
-            self._callback()
+        for m in self._on_status_update:
+            await m()
 
-    def get_categories(self):
+    def get_categories(self) -> list:
         """
         Get the categories (for hass)
         """
         # COMPONENT_TYPES = ["switch", "sensor", "binary_sensor", "cover", "climate", "light"]
         return []
 
-    def on_status_update(self, meth):
-        self._on_status_update = meth
+    def on_status_update(self, meth: type) -> None:
+        self._on_status_update.append(meth)
 
 
 class Blind(Channel):
@@ -135,46 +151,49 @@ class Blind(Channel):
     """
 
     _state = None
-    _position = None
+    _position = 0
 
-    def get_categories(self):
+    def get_categories(self) -> list:
         return ["cover"]
 
-    def get_position(self):
+    def get_position(self) -> str:
         return self._position
 
-    def get_state(self):
+    def get_state(self) -> str:
         return self._state
 
-    def is_closed(self):
+    def is_closed(self) -> bool:
         if self._state == 0x02:
             return True
         return False
 
-    def is_open(self):
+    def is_open(self) -> bool:
         if self._state == 0x01:
             return True
         return False
 
-    async def open(self):
+    def support_position(self) -> bool:
+        return False
+
+    async def open(self) -> None:
         cls = commandRegistry.get_command(0x05, self._module.get_type())
         msg = cls(self._address)
         msg.channel = self._num
         await self._writer(msg)
 
-    async def close(self):
+    async def close(self) -> None:
         cls = commandRegistry.get_command(0x06, self._module.get_type())
         msg = cls(self._address)
         msg.channel = self._num
         await self._writer(msg)
 
-    async def stop(self):
+    async def stop(self) -> None:
         cls = commandRegistry.get_command(0x04, self._module.get_type())
         msg = cls(self._address)
         msg.channel = self._num
         await self._writer(msg)
 
-    async def set_position(self, position):
+    async def set_position(self, position: int) -> None:
         cls = commandRegistry.get_command(0x1C, self._module.get_type())
         msg = cls(self._address)
         msg.channel = self._num
@@ -190,25 +209,53 @@ class Button(Channel):
 
     _enabled = True
     _closed = False
-    _on = None
+    _led_state = None
 
-    def get_categories(self):
-        return ["binary_sensor"]
+    def get_categories(self) -> list:
+        return ["binary_sensor", "led"]
 
-    def is_closed(self):
+    def is_closed(self) -> bool:
         """
         Return if this button is on
         """
         return self._closed
 
-    def _callback(self):
-        self._on_status_update(self.is_closed())
+    def is_on(self) -> bool:
+        """
+        Return if this relay is on
+        """
+        if self._led_state == "on":
+            return True
+        return False
+
+    async def set_led_state(self, state: str) -> None:
+        """
+        Set led
+
+        :return: None
+        """
+        if state == "on":
+            code = 0xF6
+        elif state == "slow":
+            code = 0xF7
+        elif state == "fast":
+            code = 0xF8
+        elif state == "off":
+            code = 0xF5
+        else:
+            return
+        cls = commandRegistry.get_command(code, self._module.get_type())
+        msg = cls(self._address)
+        msg.leds = [self._num]
+        await self._writer(msg)
+        await self.update({"led_state": state})
 
 
 class ButtonCounter(Button):
     """
     A ButtonCounter channel
     This channel can act as a button and as a counter
+    HASS OK
     """
 
     _Unit = None
@@ -216,23 +263,17 @@ class ButtonCounter(Button):
     _counter = None
     _delay = None
 
-    def get_categories(self):
+    def get_categories(self) -> list:
         if self._counter:
             return ["sensor"]
         return ["binary_sensor"]
 
-    def get_class(self):
+    def is_counter_channel(self) -> bool:
         if self._counter:
-            return "counter"
-        return None
+            return True
+        return False
 
-    def get_counter_unit(self):
-        return ENERGY_KILO_WATT_HOUR
-
-    def get_unit(self):
-        return "W"
-
-    def get_state(self):
+    def get_state(self) -> int:
         val = 0
         # if we don't know the delay
         # or we don't know the unit
@@ -250,41 +291,146 @@ class ButtonCounter(Button):
             val = 0
         return round(val, 2)
 
-    def get_counter_state(self):
-        return self._counter
+    def get_unit(self) -> str:
+        return "W"
 
-    def _callback(self):
-        self._on_status_update(self.get_state())
+    def get_counter_state(self) -> int:
+        return round((self._counter / self._pulses), 2)
+
+    def get_counter_unit(self) -> str:
+        return ENERGY_KILO_WATT_HOUR
+
+
+class Sensor(Button):
+    """
+    A Sensor channel
+    HASS OK
+    This is a bit wierd, but this happens because of code sharing with openhab
+    A sensor in this case is actually a Button
+    """
+
+
+class ThermostatChannel(Button):
+    """
+    A Thermostat channel
+    These are the booster/heater/alarms
+    HASS OK
+    """
 
 
 class Dimmer(Channel):
     """
     A Dimmer channel
+    HASS OK
     """
 
-    def get_categories(self):
+    _state: int = 0
+
+    def get_categories(self) -> list:
         return ["light"]
 
+    def is_on(self) -> bool:
+        """
+        Check if a dimmer is turned on
+        """
+        if self._state == 0:
+            return False
+        return True
 
-class EdgeLit(Channel):
+    def get_dimmer_state(self) -> int:
+        """
+        Return the dimmer state
+        """
+        return self._state
+
+    async def set_dimmer_state(self, slider, transitiontime=0) -> None:
+        """
+        Set dimmer to slider
+        """
+        cls = commandRegistry.get_command(0x07, self._module.get_type())
+        msg = cls(self._address)
+        msg.dimmer_state = slider
+        msg.dimmer_transitiontime = int(transitiontime)
+        msg.dimmer_channels = [self._num]
+        await self._writer(msg)
+
+    async def restore_dimmer_state(self, transitiontime=0) -> None:
+        """
+        restore dimmer to last known state
+        """
+        cls = commandRegistry.get_command(0x11, self._module.get_type())
+        msg = cls(self._address)
+        msg.dimmer_transitiontime = int(transitiontime)
+        msg.dimmer_channels = [self._num]
+        await self._writer(msg)
+
+
+class Temperature(Channel):
     """
-    An EdgeLit channel
+    A Temperature sensor channel
+    HASS OK
     """
 
+    _cur = 0
+    _max = None
+    _min = None
+
+    def get_categories(self) -> list:
+        return ["sensor"]
+
+    def get_class(self) -> str:
+        return DEVICE_CLASS_TEMPERATURE
+
+    def get_unit(self) -> str:
+        return TEMP_CELSIUS
+
+    def get_state(self) -> int:
+        return round(self._cur, 2)
+
+    def is_temperature(self) -> bool:
+        return True
+
+
+class SensorNumber(Channel):
+    """
+    A Numeric Sensor channel
+    HASS OK
+    """
+
+    _cur = 0
+
     def get_categories(self):
-        return ["light"]
+        return ["sensor"]
+
+    def get_class(self):
+        return None
+
+    def get_unit(self):
+        return None
+
+    def get_state(self):
+        return round(self._cur, 2)
 
 
-class Memo(Channel):
+class LightSensor(Channel):
     """
-    A Memo text
+    A light sensor channel
+    HASS OK
     """
 
+    _cur = 0
 
-class ThermostatChannel(Channel):
-    """
-    A Thermostat channel
-    """
+    def get_categories(self):
+        return ["sensor"]
+
+    def get_class(self):
+        return DEVICE_CLASS_ILLUMINANCE
+
+    def get_unit(self):
+        return None
+
+    def get_state(self):
+        return round(self._cur, 2)
 
 
 class Relay(Channel):
@@ -295,16 +441,16 @@ class Relay(Channel):
 
     _on = None
 
-    def get_categories(self):
+    def get_categories(self) -> list:
         return ["switch"]
 
-    def is_on(self):
+    def is_on(self) -> bool:
         """
         Return if this relay is on
         """
         return self._on
 
-    async def turn_on(self):
+    async def turn_on(self) -> None:
         """
         Send the turn on message
         """
@@ -313,7 +459,7 @@ class Relay(Channel):
         msg.relay_channels = [self._num]
         await self._writer(msg)
 
-    async def turn_off(self):
+    async def turn_off(self) -> None:
         """
         Send the turn off message
         """
@@ -322,76 +468,47 @@ class Relay(Channel):
         msg.relay_channels = [self._num]
         await self._writer(msg)
 
-    def _callback(self):
-        self._on_status_update(self.is_on())
 
-
-class Sensor(Button):
+class EdgeLit(Channel):
     """
-    A Sensor channel
-    This is a bit wier, but this happens because of code sharing with openhab
-    A sensor in this case is actually a Button
+    An EdgeLit channel
+    """
+
+    # def get_categories(self):
+    #    return ["light"]
+
+
+class Memo(Channel):
+    """
+    A Memo text
     """
 
 
-class SensorNumber(Channel):
-    """
-    A Numeric Sensor channel
-    """
-
-    def get_categories(self):
-        return []
-
-    def get_state(self):
-        return None
-
-    def _callback(self):
-        self._on_status_update(self.get_state())
-
-
-class Temperature(Channel):
-    """
-    A Temperature sensor channel
-    """
-
-    _cur = None
-    _max = None
-    _min = None
-
-    def get_categories(self):
-        return ["sensor"]
-
-    def get_class(self):
-        return DEVICE_CLASS_TEMPERATURE
-
-    def get_unit(self):
-        return TEMP_CELSIUS
-
-    def get_state(self):
-        return self._cur
-
-    def _callback(self):
-        self._on_status_update(self.get_state())
-
-
-class LightSensor(Channel):
-    """
-    A light sensor channel
-    """
-
-    _cur = None
-
-    def get_categories(self):
-        return ["sensor"]
-
-    def get_class(self):
-        return None
-
-    def get_unit(self):
-        return None
-
-    def get_state(self):
-        return self._cur
-
-    def _callback(self):
-        self._on_status_update(self.get_state())
+# _mode = None
+# _target = None
+# _cur = None
+#
+# def get_categories(self):
+#    return ["climate"]
+#
+# async def set_temp(self, temp) -> None:
+#    cls = commandRegistry.get_command(0xE4, self._module.get_type())
+#    msg = cls(self._address)
+#    msg.temp = temp * 2
+#    await self._writer(msg)
+#
+# async def set_mode(self, mode) -> None:
+#    if mode == "safe":
+#        code = 0xDE
+#    elif mode == "comfort":
+#        code = 0xDB
+#    elif mode == "day":
+#        code = 0xDC
+#    elif mode == "night":
+#        code = 0xDD
+#    cls = commandRegistry.get_command(code, self._module.get_type())
+#    msg = cls(self._address)
+#    await self._writer(msg)
+#
+# def get_state(self) -> int:
+#    return round(self._cur, 2)
