@@ -1,6 +1,7 @@
 """
 This represents a velbus module
 """
+from __future__ import annotations
 
 import logging
 import os
@@ -24,6 +25,7 @@ from velbusaio.channels import (
 )
 from velbusaio.const import PRIORITY_LOW
 from velbusaio.helpers import get_cache_dir, handle_match, keys_exists
+from velbusaio.messages.blind_status import BlindStatusMessage, BlindStatusNgMessage
 from velbusaio.messages.channel_name_part1 import (
     ChannelNamePart1Message,
     ChannelNamePart1Message2,
@@ -42,6 +44,8 @@ from velbusaio.messages.channel_name_part3 import (
 from velbusaio.messages.channel_name_request import ChannelNameRequestMessage
 from velbusaio.messages.clear_led import ClearLedMessage
 from velbusaio.messages.counter_status import CounterStatusMessage
+from velbusaio.messages.dimmer_channel_status import DimmerChannelStatusMessage
+from velbusaio.messages.dimmer_status import DimmerStatusMessage
 from velbusaio.messages.fast_blinking_led import FastBlinkingLedMessage
 from velbusaio.messages.memory_data import MemoryDataMessage
 from velbusaio.messages.module_status import (
@@ -57,6 +61,7 @@ from velbusaio.messages.read_data_from_memory import ReadDataFromMemoryMessage
 from velbusaio.messages.relay_status import RelayStatusMessage
 from velbusaio.messages.sensor_temperature import SensorTemperatureMessage
 from velbusaio.messages.set_led import SetLedMessage
+from velbusaio.messages.slider_status import SliderStatusMessage
 from velbusaio.messages.slow_blinking_led import SlowBlinkingLedMessage
 from velbusaio.messages.temp_sensor_status import TempSensorStatusMessage
 from velbusaio.messages.update_led_status import UpdateLedStatusMessage
@@ -223,17 +228,21 @@ class Module:
             # self._cstatus = message.status_str
         elif isinstance(message, PushButtonStatusMessage):
             for channel in message.closed:
+                channel = self._translate_channel_name(channel)
                 await self._channels[channel].update({"closed": True})
             for channel in message.opened:
+                channel = self._translate_channel_name(channel)
                 await self._channels[channel].update({"closed": False})
         elif isinstance(message, ModuleStatusMessage):
             for channel in self._channels.keys():
+                channel = self._translate_channel_name(channel)
                 if channel in message.closed:
                     await self._channels[channel].update({"closed": True})
                 elif isinstance(self._channels[channel], (Button, ButtonCounter)):
                     await self._channels[channel].update({"closed": False})
         elif isinstance(message, ModuleStatusMessage2):
             for channel in self._channels.keys():
+                channel = self._translate_channel_name(channel)
                 if channel in message.closed:
                     await self._channels[channel].update({"closed": True})
                 elif isinstance(self._channels[channel], (Button, ButtonCounter)):
@@ -245,7 +254,8 @@ class Module:
         elif isinstance(message, CounterStatusMessage) and isinstance(
             self._channels[message.channel], ButtonCounter
         ):
-            await self._channels[message.channel].update(
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update(
                 {
                     "pulses": message.pulses,
                     "counter": message.counter,
@@ -256,6 +266,7 @@ class Module:
             await self._channels[99].update({"cur": message.light_value})
         elif isinstance(message, UpdateLedStatusMessage):
             for channel in self._channels.keys():
+                channel = self._translate_channel_name(message.channel)
                 if channel in message.led_slow_blinking:
                     await self._channels[channel].update({"led_state": "slow"})
                 if channel in message.led_fast_blinking:
@@ -271,19 +282,41 @@ class Module:
         elif isinstance(message, SetLedMessage):
             for channel in self._channels.keys():
                 if channel in message.leds:
+                    channel = self._translate_channel_name(channel)
                     await self._channels[channel].update({"led_state": "on"})
         elif isinstance(message, ClearLedMessage):
             for channel in self._channels.keys():
                 if channel in message.leds:
+                    channel = self._translate_channel_name(channel)
                     await self._channels[channel].update({"led_state": "off"})
         elif isinstance(message, SlowBlinkingLedMessage):
             for channel in self._channels.keys():
                 if channel in message.leds:
+                    channel = self._translate_channel_name(channel)
                     await self._channels[channel].update({"led_state": "slow"})
         elif isinstance(message, FastBlinkingLedMessage):
             for channel in self._channels.keys():
                 if channel in message.leds:
+                    channel = self._translate_channel_name(channel)
                     await self._channels[channel].update({"led_state": "fast"})
+        elif isinstance(message, DimmerChannelStatusMessage):
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update({"state": message.cur_dimmer_state()})
+        elif isinstance(message, SliderStatusMessage):
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update({"state": message.cur_slider_state()})
+        elif isinstance(message, DimmerStatusMessage):
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update({"state": message.cur_dimmer_state()})
+        elif isinstance(message, BlindStatusNgMessage):
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update(
+                {"state": message.status, "position": message.position}
+            )
+        elif isinstance(message, BlindStatusMessage):
+            channel = self._translate_channel_name(message.channel)
+            await self._channels[channel].update({"state": message.status})
+
         self._cache()
 
     def get_channels(self) -> list:
